/**
 * Exercises the Lexicon class via a fairly straightforward
 * unit test.
 */

#include <iostream>
#include <fstream>
#include <string>

#include "console.h"
#include "lexicon.h"
#include "simpio.h"
using namespace std;

static void loadWordsFromFile(Lexicon& lexicon, const string& filename) {
    ifstream infile(filename.c_str());
    if (infile.fail()) error("\"" + filename + "\" could not be found.");
    
    while (true) {
        string word;
        getline(infile, word);
        if (infile.fail()) return;
        lexicon.add(word);
    }
}

static void testLexicon(Lexicon& lexicon) {
    while (true) {
        string response = getLine("Enter a word [or hit return to quit]: ");
        if (response.empty()) return;
        if (lexicon.contains(response)) {
            cout << "\"" << response << "\" is a word." << endl;
        } else if (lexicon.containsPrefix(response)) {
            cout << "\"" << response << "\" isn't a word, but it's "
                 << "a prefix of one or more words." << endl;
        } else {
            cout << "\"" << response << "\" is neither a word nor "
                 << "a prefix of some word." << endl;
        }
    }
}

static const string kWordsFile("words.txt");
int main() {
    Lexicon lexicon;
    loadWordsFromFile(lexicon, kWordsFile);
    testLexicon(lexicon);
    return 0;
}
