/**
 * Presents a trie-backed implementation of a Lexicon
 * that supports add, remove, contains, containsPrefix.
 */

#include "lexicon.h"
using namespace std;

struct node {
    bool isWord;
    Map<char, node *> suffixes;
    
    node();
    ~node();
};

node::node() {
    isWord = false;
}

node::~node() {
    foreach (char ch in suffixes) {
        delete suffixes[ch];
    }
}

Lexicon::Lexicon() {
    root = NULL;
}

Lexicon::~Lexicon() {
    delete root;
}

void Lexicon::add(const string& word) {
    ensureNodeExists(word)->isWord = true;
}

bool Lexicon::containsPrefix(const string& prefix) const {
    return findNode(prefix) != NULL;
}

bool Lexicon::contains(const string& word) const {
    const node *found = findNode(word);
    return found != NULL && found->isWord;
}

const node *Lexicon::findNode(const string& str) const {
    const node *curr = root;
    for (int pos = 0; pos < str.size() && curr != NULL; pos++) {
        curr = curr->suffixes.containsKey(str[pos]) ?
            curr->suffixes.get(str[pos]) : NULL;
    }
    
    return curr;
}

node *Lexicon::ensureNodeExists(const string& str) {
    node **currp = &root;
    int pos = 0;
    while (true) {
        if (*currp == NULL) *currp = new node;
        if (pos == str.size()) break;
        currp = &((*currp)->suffixes[str[pos]]);
        pos++;
    }
    return *currp;
}
